/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef _SOLARIS_CMDLIST_H
#define	_SOLARIS_CMDLIST_H

#pragma ident	"%Z%%M%	%I%	%E% SMI"

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * List builtins for Solaris.
 * The list here is partially autogenerated and partially hand-picked
 * based on compatibility with the native Solaris versions of these
 * tools
 */

/* POSIX compatible commands */
#ifdef _NOT_YET
#define	XPG6CMDLIST(f)   { "/usr/xpg6/bin/" #f, NV_BLTIN|NV_NOFREE, bltin(f) },
#define	XPG4CMDLIST(f)   { "/usr/xpg4/bin/" #f, NV_BLTIN|NV_NOFREE, bltin(f) },
#else
#define	XPG6CMDLIST(f)
#define	XPG4CMDLIST(f)
#endif /* NOT_YET */
/*
 * Commands which are 100% compatible with native Solaris versions (/bin is
 * a softlink to ./usr/bin so both need to be listed here)
 */
#define	BINCMDLIST(f)	 { "/bin/" #f,  	NV_BLTIN|NV_NOFREE, bltin(f) },
/* Make all ksh93 builtins accessible when /usr/ast/bin was added to ${PATH} */
#define	ASTCMDLIST(f)	 { "/usr/ast/bin/" #f,  NV_BLTIN|NV_NOFREE, bltin(f) },

/* undo ast_map.h #defines to avoid collision */
#undef basename
#undef dirname

/* Generated data, do not edit. */
XPG4CMDLIST(basename)
ASTCMDLIST(basename)
BINCMDLIST(cat)
ASTCMDLIST(cat)
XPG4CMDLIST(chgrp)
ASTCMDLIST(chgrp)
ASTCMDLIST(chmod)
XPG4CMDLIST(chown)
BINCMDLIST(chown)
ASTCMDLIST(chown)
ASTCMDLIST(cmp)
ASTCMDLIST(comm)
XPG4CMDLIST(cp)
ASTCMDLIST(cp)
ASTCMDLIST(cut)
XPG4CMDLIST(date)
ASTCMDLIST(date)
ASTCMDLIST(dirname)
XPG4CMDLIST(expr)
ASTCMDLIST(expr)
ASTCMDLIST(fds)
ASTCMDLIST(fmt)
ASTCMDLIST(fold)
BINCMDLIST(head)
ASTCMDLIST(head)
XPG4CMDLIST(id)
ASTCMDLIST(id)
ASTCMDLIST(join)
XPG4CMDLIST(ln)
ASTCMDLIST(ln)
ASTCMDLIST(logname)
BINCMDLIST(mkdir)
ASTCMDLIST(mkdir)
ASTCMDLIST(mkfifo)
XPG4CMDLIST(mv)
ASTCMDLIST(mv)
ASTCMDLIST(paste)
ASTCMDLIST(pathchk)
ASTCMDLIST(rev)
XPG4CMDLIST(rm)
ASTCMDLIST(rm)
BINCMDLIST(rmdir)
ASTCMDLIST(rmdir)
XPG4CMDLIST(stty)
ASTCMDLIST(stty)
XPG4CMDLIST(tail)
ASTCMDLIST(tail)
BINCMDLIST(tee)
ASTCMDLIST(tee)
ASTCMDLIST(tty)
ASTCMDLIST(uname)
BINCMDLIST(uniq)
ASTCMDLIST(uniq)
BINCMDLIST(wc)
ASTCMDLIST(wc)
/* End-of-generated-data. */

/* Mandatory for ksh93 test suite and AST scripts */
BINCMDLIST(getconf)

#ifdef	__cplusplus
}
#endif

#endif /* _SOLARIS_CMDLIST_H */
